package com.gic.haoban.manage.service.service.impl;

import java.util.Date;
import java.util.List;

import com.gic.haoban.base.api.common.Constant;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.commons.util.EntityUtil;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.common.utils.StringUtil;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.service.dao.mapper.DepartmentMapper;
import com.gic.haoban.manage.service.entity.TabHaobanDepartment;
import com.gic.haoban.manage.service.service.DepartmentService;
import com.github.pagehelper.Page;

@Service
public class DepartmentServiceImpl implements DepartmentService {
	
	@Autowired
	private DepartmentMapper mapper;

	@Override
	public List<TabHaobanDepartment> listByParentId(String parentId) {
		
		return mapper.listByParentId(parentId);
	}

	@Override
	public TabHaobanDepartment selectById(String departmentId) {
		
		return mapper.selectByPrimaryKey(departmentId);
	}

	@Override
	public String add(DepartmentDTO department) {
		Date now = new Date();
		TabHaobanDepartment tab = EntityUtil.changeEntityByJSON(TabHaobanDepartment.class, department);
        tab.setDepartmentId(StringUtil.randomUUID());
        Integer sort = tab.getSort();
        if(sort == null) {
            int maxSort = mapper.selectMaxSort(department.getParentDepartmentId());
            tab.setSort(maxSort + 1);

        }
        if (StringUtils.isBlank(department.getChainId())) {
            TabHaobanDepartment pDepartment = this.selectById(department.getParentDepartmentId());
            if (null == pDepartment) {
                throw new RuntimeException("父部门不存在");
            }
            tab.setChainId(pDepartment.getChainId() + Constant.ID_SEPARATOR + pDepartment.getDepartmentId());
			tab.setChainName(pDepartment.getChainName() + Constant.NAME_SEPARATOR + pDepartment.getDepartmentName());
			tab.setLevel(pDepartment.getLevel() + 1);

        }
		Integer isStore = department.getIsStore();
		if(isStore == null){
			department.setIsStore(0);
		}
		tab.setCreateTime(now);
		tab.setUpdateTime(now);
		tab.setStatusFlag(1);
		tab.setRecycleFlag(0);
		if (department.getTempFlag() == null) {
			department.setTempFlag(0);
		}

        mapper.insert(tab);
		return tab.getDepartmentId();
		
	}

	@Override
	public void edit(DepartmentDTO department) {
		Date now = new Date();
		TabHaobanDepartment tab = EntityUtil.changeEntityByJSON(TabHaobanDepartment.class, department);
		tab.setUpdateTime(now);
		mapper.updateByPrimaryKeySelective(tab);
		
	}

	@Override
	public void del(String departmentId) {
		TabHaobanDepartment tab = mapper.selectByPrimaryKey(departmentId);
		if(tab != null) {
			tab.setStatusFlag(0);
			tab.setUpdateTime(new Date());
		}
		mapper.updateByPrimaryKeySelective(tab);
		
	}

	@Override
	public TabHaobanDepartment selectByRelatedId(String relatedId) {
		List<TabHaobanDepartment> list = mapper.selectByRelatedId(relatedId);
		if(CollectionUtils.isNotEmpty(list)){
			return list.get(0);
		}
		return null;
	}

	@Override
	public TabHaobanDepartment getByWxId(String wxDepartmentId, String wxEnterpriseId) {
		List<TabHaobanDepartment> list = this.mapper.getByWxId(wxDepartmentId, wxEnterpriseId);
		if(CollectionUtils.isNotEmpty(list)){
			return list.get(0);
		}
		return null;
	}

	@Override
	public void recycle(String departmentId) {
		TabHaobanDepartment tab = mapper.selectByPrimaryKey(departmentId);
		if(tab != null) {
			tab.setRecycleFlag(1);
			tab.setUpdateTime(new Date());
		}
		mapper.updateByPrimaryKeySelective(tab);
	}

	@Override
	public Page<TabHaobanDepartment> pageDepartmentByParams(String wxEnterpriseId, String keyword, Integer storeFlag, Integer recycleFlag) {
		
		return mapper.pageDepartmentByParams(wxEnterpriseId,keyword,storeFlag,recycleFlag);
	}

	@Override
	public void repairRecycle(TabHaobanDepartment tab) {
		if(tab != null) {
			tab.setRecycleFlag(0);
			tab.setUpdateTime(new Date());
		}
		mapper.updateByPrimaryKeySelective(tab);
		
	}

	@Override
	public DepartmentDTO getRootByEnterpriseId(String wxEnterpriseId) {
		TabHaobanDepartment tab = mapper.getRootByEnterpriseId(wxEnterpriseId);
		return EntityUtil.changeEntityByJSON(DepartmentDTO.class, tab);
	}

	@Override
	public DepartmentDTO getByRelatedId(String relatedId) {
		TabHaobanDepartment tab = mapper.getByRelatedId(relatedId);
		return EntityUtil.changeEntityByJSON(DepartmentDTO.class, tab);
	}

	@Override
	public Page<TabHaobanDepartment> pageByParentId(String parentId, String keyword) {
		return mapper.pageByParentId(parentId,keyword);
	}

	@Override
	public List<TabHaobanDepartment> listByChainId(String chainId, String wxEnterpriseId) {
		
		return mapper.listByChainId(chainId,wxEnterpriseId);
	}

	@Override
	public Integer departmentRecycleCount(String wxEnterpriseId) {
		return mapper.departmentRecycleCount(wxEnterpriseId);
	}

	@Override
	public int countSyncDepartmentByEnterpriseId(String enterpriseId) {
		return mapper.countSyncDepartmentByEnterpriseId(enterpriseId);
	}

	@Override
	public int countByDepartment(String departmentId) {
		return mapper.countByDepartment(departmentId);
	}

	@Override
	public DepartmentDTO getByRelatedIdNotInRecycle(String relatedId) {
		return EntityUtil.changeEntityByJSON(DepartmentDTO.class,mapper.getByRelatedIdNotInRecycle(relatedId));
	}

	@Override
	public List<DepartmentDTO> listByIds(List<String> departmentIds) {
		return EntityUtil.changeEntityListByJSON(DepartmentDTO.class, mapper.listByIds(departmentIds));
	}

	@Override
	public DepartmentDTO getTempFlagDepartment(String wxEnterpriseId) {
		return EntityUtil.changeEntityByJSON(DepartmentDTO.class, mapper.getTempFlagDepartment(wxEnterpriseId));
	}

	@Override
	public Integer totalStoreCountByEnterpriseId(String wxEnterpriseId) {
		return mapper.totalStoreCountByEnterpriseId(wxEnterpriseId);
	}
}
